import os
from lxml import etree
import pytest
from conftest import BASE_DIR

@pytest.fixture
def voila_args():
    nb_path = os.path.join(BASE_DIR, 'nb_report.ipynb')
    return [nb_path, '--VoilaTest.config_file_paths=[]']


async def test_report_view(http_server_client, base_url):
    response = await http_server_client.fetch(base_url)
    assert response.code == 200
    html_body = response.body.decode('utf-8')

    parser = etree.HTMLParser()
    tree = etree.fromstring(html_body, parser=parser)

    assert "SVG circles" in html_body
    assert "Voilà logo" not in html_body

    caption_tag = tree.xpath("//p[contains(text(), 'SVG circles')]")
    assert caption_tag

    header = tree.xpath("//*[contains(text(), 'Graphics')]")
    assert header

    circle_tag = tree.xpath("//svg/circle")
    assert circle_tag

    voila_caption_tag = tree.xpath("//p[contains(text(), 'Voilà logo')]")
    assert not voila_caption_tag

    img_tag = tree.xpath("//img")
    assert not img_tag

    # test element order
    ordered_tags = [header[0], circle_tag[0], caption_tag[0]]
    document_order = [e for e in tree.getiterator() if e in ordered_tags]
    assert ordered_tags == document_order



import os
import voila.app
import pytest

BASE_DIR = os.path.dirname(__file__)


class VoilaTest(voila.app.Voila):
    def listen(self):
        pass  # the ioloop is taken care of by the pytest-tornado framework


@pytest.fixture
def voila_app(voila_args, voila_config):
    voila_app = VoilaTest.instance()
    voila_app.initialize(voila_args + ['--no-browser', '--template=gridstack'])
    voila_config(voila_app)
    voila_app.start()
    yield voila_app
    voila_app.stop()
    voila_app.clear_instance()


@pytest.fixture
def base_url():
    return "/"


@pytest.fixture
def app(voila_app):
    return voila_app.app


@pytest.fixture
def voila_config():
    return lambda app: None

@pytest.fixture
def voila_args():
    nb_path = os.path.join(BASE_DIR, 'nb.ipynb')
    return [nb_path, '--VoilaTest.config_file_paths=[]']



import os
import pytest
from lxml import etree
import json
import tempfile

BASE_DIR = os.path.dirname(__file__)

@pytest.fixture(
    params=[None, 'grid_default', 'report_default'],
    ids="activeView={}".format)
def activeView(request):
    return request.param

@pytest.fixture(params=[None, 20], ids="defaultCellHeight={}".format)
def defaultCellHeight(request):
    return request.param


@pytest.fixture(params=[None, 0, 10], ids="cellMargin={}".format)
def cellMargin(request):
    return request.param


@pytest.fixture(params=[None, 15], ids="maxColumns={}".format)
def maxColumns(request):
    return request.param


@pytest.fixture
def nb_metadata(defaultCellHeight, cellMargin, maxColumns, activeView):
    grid_default = {"name": "grid", "type": "grid"}
    if defaultCellHeight is not None:
        grid_default['defaultCellHeight'] = defaultCellHeight
    if cellMargin is not None:
        grid_default['cellMargin'] = cellMargin
    if maxColumns is not None:
        grid_default['maxColumns'] = maxColumns
    activeView_dict = {"activeView": activeView} if activeView else {}
    return {"extensions": {
               "jupyter_dashboards":  {
                   "views": {
                       "grid_default": grid_default},
                   **activeView_dict
                   }}}


@pytest.fixture
def voila_args(nb_metadata):
    nb_path = os.path.join(BASE_DIR, 'nb.ipynb')
    with open(nb_path) as fid:
        nb_json = json.load(fid)
    nb_json['metadata'] = nb_metadata
    nb_temp = tempfile.NamedTemporaryFile(delete=False, mode="w", suffix=".ipynb")
    with nb_temp as fid:
        json.dump(nb_json, fid)
    print(nb_temp.name)

    return [nb_temp.name, '--VoilaTest.config_file_paths=[]']


async def test_gridstack_general_conf(http_server_client, base_url, nb_metadata):
    response = yield http_server_client.fetch(base_url)
    assert response.code == 200
    html_body = response.body.decode('utf-8')

    parser = etree.HTMLParser()
    tree = etree.fromstring(html_body, parser=parser)
    gridstack_config = tree.xpath("//script[contains(text(), 'gridstack')]")[0].text

    grid_default = nb_metadata['extensions']['jupyter_dashboards']['views']['grid_default']
    activeView = nb_metadata['extensions']['jupyter_dashboards'].get('activeView')

    def assert_config(metadata_name, gridstack_property):
        value = grid_default.get(metadata_name)
        if value is None:
            assert gridstack_property not in gridstack_config
        else:
            assert (f"{gridstack_property}: {value}" in gridstack_config)

    if activeView is None or activeView == 'grid_default':

        assert_config('defaultCellHeight', 'cellHeight')
        assert_config('cellMargin', 'margin')
        assert_config('maxColumns', 'column')
    else:
        assert 'cellHeight' not in gridstack_config
        assert 'margin' not in gridstack_config
        assert 'column' not in gridstack_config



import os
import pytest
from lxml import etree

BASE_DIR = os.path.dirname(__file__)


async def test_template_test(http_server_client, base_url):
    response = await http_server_client.fetch(base_url)
    assert response.code == 200
    html_body = response.body.decode('utf-8')

    parser = etree.HTMLParser()
    tree = etree.fromstring(html_body, parser=parser)

    # test top-level section
    elem = tree.xpath("//section[@id='demo']")
    assert elem

    # test width/height params
    elem = tree.xpath("//pre[text()='Hi !\n']/ancestor::div[@class='grid-stack-item']")[0]
    assert elem.attrib['gs-w'] == '6'
    assert elem.attrib['gs-h'] == '5'
    assert elem.attrib['gs-x'] == '0'
    assert elem.attrib['gs-y'] == '0'

    # test cell "2"
    elem = tree.xpath("//pre[text()='2']/ancestor::div[@class='grid-stack-item']")[0]
    assert elem.attrib['gs-w'] == '4'
    assert elem.attrib['gs-h'] == '4'
    assert elem.attrib['gs-x'] == '6'
    assert elem.attrib['gs-y'] == '0'

    # markdown cell
    elem = tree.xpath("//h1[text()='This is markdown']/ancestor::div[@class='grid-stack-item']")[0]
    assert elem.attrib['gs-w'] == '10'
    assert elem.attrib['gs-h'] == '2'
    assert elem.attrib['gs-x'] == '0'
    assert elem.attrib['gs-y'] == '5'

    # test hidden cell
    elem = tree.xpath("//*[text()='This is a hidden cell.']")
    assert not elem

    # test absence of handles
    elem = tree.xpath("//div[contains(@class, 'gridhandle')]")
    assert not elem

    assert "cellHeight: 40" in html_body
    assert "column: 12" in html_body
    assert "margin: 10" in html_body



#############################################################################
# Copyright (c) 2018, Voila Contributors                                    #
# Copyright (c) 2018, QuantStack                                            #
#                                                                           #
# Distributed under the terms of the BSD 3-Clause License.                  #
#                                                                           #
# The full license is in the file LICENSE, distributed with this software.  #
#############################################################################

import json
import os.path as osp
from .server_extension import load_jupyter_server_extension  # noqa

__version__ = "0.3.0a1"

HERE = osp.abspath(osp.dirname(__file__))

with open(osp.join(HERE, 'labextension', 'package.json')) as fid:
    data = json.load(fid)


def _jupyter_labextension_paths():
    return [{
        'src': 'labextension',
        'dest': data['name']
    }]


def _jupyter_nbextension_paths():
    return [dict(
        section="notebook",
        src="static",
        dest="voila-gridstack",
        require="voila-gridstack/extension")]



#############################################################################
# Copyright (c) 2018, Voila Contributors                                    #
# Copyright (c) 2018, QuantStack                                            #
#                                                                           #
# Distributed under the terms of the BSD 3-Clause License.                  #
#                                                                           #
# The full license is in the file LICENSE, distributed with this software.  #
#############################################################################

from jupyter_server.utils import url_path_join

from voila.paths import collect_template_paths
from voila.handler import VoilaHandler
from voila.configuration import VoilaConfiguration


def load_jupyter_server_extension(server_app):
    """ Extension to create an URL for voila-dashboard using GridStack, to be able to call it from Jupyter Notebook

    """

    web_app = server_app.web_app

    voila_configuration = VoilaConfiguration(parent=server_app)
    voila_configuration.template = "gridstack"
    voila_configuration.config.VoilaConfiguration.resources = {"gridstack": {"show_handles": False, "extension": True}}

    template_paths = collect_template_paths(
        ['voila', 'nbconvert'],
        voila_configuration.template
    )

    host_pattern = '.*$'
    base_url = url_path_join(web_app.settings['base_url'])

    web_app.add_handlers(host_pattern, [
        (url_path_join(base_url, '/voila/dashboard/(.*)'), VoilaHandler, {
            'config': server_app.config,
            'template_paths': template_paths,
            'voila_configuration': voila_configuration
        }),
    ])




import os
import pytest

from lxml import etree

BASE_DIR = os.path.dirname(__file__)

@pytest.fixture(params=[False, True], ids="show_handles={}".format)
def show_handles(request):
    return request.param

@pytest.fixture
def voila_resources(show_handles):
    return {"gridstack": {"show_handles": show_handles}}

@pytest.fixture
def voila_args(voila_resources):
    nb_path = os.path.join(BASE_DIR, 'nb.ipynb')
    return [nb_path,
            '--VoilaTest.config_file_paths=[]',
            '--VoilaConfiguration.resources={}'.format(voila_resources)]


async def test_resources_gridstack_show_handles(http_server_client, base_url, show_handles):
    """Test with handlebars"""
    response = await http_server_client.fetch(base_url)
    assert response.code == 200
    html_body = response.body.decode('utf-8')

    parser = etree.HTMLParser()
    tree = etree.fromstring(html_body, parser=parser)

    elem = tree.xpath("//div[@class='gridhandle']")
    assert (show_handles and elem) or (not show_handles and not elem)

    gridstack_config = tree.xpath("//script[contains(text(), 'GridStack')]")[0].text

    if show_handles:
        assert "handles: 'e, se, s, sw, w'" in gridstack_config
    else:
        assert "handles: 'none'" in gridstack_config



import os
import pytest
from conftest import BASE_DIR
from lxml import etree

@pytest.fixture
def voila_args():
    nb_path = os.path.join(BASE_DIR, 'nb_without_metadata.ipynb')
    return [nb_path, '--VoilaTest.config_file_paths=[]']


async def test_render_without_metadata(http_server_client, base_url):
    response = await http_server_client.fetch(base_url)
    assert response.code == 200
    html_body = response.body.decode('utf-8')

    parser = etree.HTMLParser()
    tree = etree.fromstring(html_body, parser=parser)

    text_elem = tree.xpath("//pre[text()='Hi !\n']")
    assert text_elem

    hidden_elem = tree.xpath("//*[text()='This is a hidden cell.']")
    assert hidden_elem

    expr_elem = tree.xpath("//pre[text()='2']")
    assert expr_elem

    md_elem = tree.xpath("//h1[text()='This is markdown']")
    assert md_elem

    all_elements = text_elem + hidden_elem + expr_elem + md_elem

    for elem in all_elements:
        attribs = elem.xpath("ancestor::div[@class='grid-stack-item']")[0].attrib
        assert attribs['gs-auto-position'] == 'true'
        assert 'gs-x' not in attribs
        assert 'gs-y' not in attribs
        assert attribs['gs-w'] == '12'
        assert attribs['gs-h'] == '2'

    # check if the document is properly ended
    assert "</html>" in html_body



"""
voila-gridstack setup
"""

import setuptools
from pathlib import Path

HERE = Path(__file__).parent.resolve()

# The name of the project
NAME = "voila-gridstack"
PACKAGE = NAME.replace('-', '_')

labext_name = "@voila-dashboards/jupyterlab-gridstack"
lab_extension_dest = (HERE / PACKAGE / "labextension")

# Representative files that should exist after a successful build
ensured_targets = [
    str(lab_extension_dest / "package.json"),
    str(lab_extension_dest / "static" / "style.js"),
    str(HERE / "share/jupyter/nbconvert/templates/gridstack/gridstack.js.j2")
]

data_files_spec = [
    ("etc/jupyter/jupyter_server_config.d", "etc/jupyter/jupyter_server_config.d", "voila-gridstack.json"),
    ("etc/jupyter/jupyter_notebook_config.d", "etc/jupyter/jupyter_notebook_config.d", "voila-gridstack.json"),
    ("etc/jupyter/nbconfig/notebook.d", "etc/jupyter/nbconfig/notebook.d", "voila-gridstack.json"),
    ("share/jupyter/nbextensions/voila-gridstack", f"{PACKAGE}/static", "**"),
    ("share/jupyter/labextensions/%s" % labext_name, str(lab_extension_dest), "**"),
    ("share/jupyter/labextensions/%s" % labext_name, HERE, "install.json"),
    ("share/jupyter/nbconvert/templates/gridstack", "share/jupyter/nbconvert/templates/gridstack", "**")
]

try:
    from jupyter_packaging import (
        wrap_installers,
        npm_builder,
        get_data_files
    )

    post_develop = npm_builder(build_cmd='install:dev', npm='jlpm', force=True)
    cmdclass = wrap_installers(post_develop=post_develop, ensured_targets=ensured_targets)
    setup_args = dict(cmdclass=cmdclass, data_files=get_data_files(data_files_spec))
except ImportError:
    setup_args = dict()

if __name__ == "__main__":
    setuptools.setup(**setup_args)
